from PIL import Image
import colorsys

def filtered_hsv(im, filter_h, filter_s, filter_v, default_col=(255,255,255)):
    """ https://stackoverflow.com/a/22237709/183995 """

    hsv_im = im.convert('HSV')
    H, S, V = 0, 1, 2
    hsv = hsv_im.split()
    mask_h = hsv[H].point(filter_h)
    mask_s = hsv[S].point(filter_s)
    mask_v = hsv[V].point(filter_v)

    Mdat = []
    for h, s, v in zip(mask_h.getdata(), mask_s.getdata(), mask_v.getdata()):
        Mdat.append(255 if (h and s and v) else 0)

    mask = mask_h
    mask.putdata(Mdat)

    filtered_im = Image.new("RGB", im.size, color=default_col)
    filtered_im.paste(im, None, mask)
    return filtered_im




from builtins import bytes

import struct
import logging

import packbits
from PIL import Image
import io

from .devicedependent import models, \
                             min_max_feed, \
                             min_max_length_dots, \
                             number_bytes_per_row, \
                             right_margin_addition, \
                             compressionsupport, \
                             cuttingsupport, \
                             expandedmode, \
                             two_color_support, \
                             modesetting

from . import BrotherQLError, BrotherQLUnsupportedCmd, BrotherQLUnknownModel, BrotherQLRasterError

try:
    from io import BytesIO
except: # Py2
    from cStringIO import StringIO as BytesIO

logger = logging.getLogger(__name__)

class BrotherQLRaster(object):

    def __init__(self, model='QL-500'):
        if model not in models:
            raise BrotherQLUnknownModel()
        self.model = model
        self.data = b''
        self._pquality = True
        self.page_number = 0
        self.cut_at_end = True
        self.dpi_600 = False
        self.two_color_printing = False
        self._compression = False
        self.exception_on_warning = False

    def _warn(self, problem, kind=BrotherQLRasterError):
        """
        Logs the warning message `problem` or raises a
        `BrotherQLRasterError` exception (changeable via `kind`)
        if `self.exception_on_warning` is set to True.

        :raises BrotherQLRasterError: Or other exception \
            set via the `kind` keyword argument.
        """
        if self.exception_on_warning:
            raise kind(problem)
        else:
            logger.warning(problem)

    def unsupported(self, problem):
        """
        Raises BrotherQLUnsupportedCmd if
        exception_on_warning is set to True.
        Issues a logger warning otherwise.

        :raises BrotherQLUnsupportedCmd:
        """
        self._warn(problem, kind=BrotherQLUnsupportedCmd)

    @property
    def two_color_support(self):
        return self.model in two_color_support

    def add_initialize(self):
        self.page_number = 0
        self.data += b'\x1B\x40' # ESC @

    def add_status_information(self):
        """ Status Information Request """
        self.data += b'\x1B\x69\x53' # ESC i S

    def add_switch_mode(self):
        """
        Switch dynamic command mode
        Switch to the raster mode on the printers that support
        the mode change (others are in raster mode already).
        """
        if self.model not in modesetting:
            self.unsupported("Trying to switch the operating mode on a printer that doesn't support the command.")
            return
        self.data += b'\x1B\x69\x61\x01' # ESC i a

    def add_invalidate(self):
        """ clear command buffer """
        self.data += b'\x00' * 200

    @property
    def mtype(self): return self._mtype

    @property
    def mwidth(self): return self._mwidth

    @property
    def mlength(self): return self._mlength

    @property
    def pquality(self): return self._pquality

    @mtype.setter
    def mtype(self, value):
        self._mtype = bytes([value & 0xFF])

    @mwidth.setter
    def mwidth(self, value):
        self._mwidth = bytes([value & 0xFF])

    @mlength.setter
    def mlength(self, value):
        self._mlength = bytes([value & 0xFF])

    @pquality.setter
    def pquality(self, value):
        self._pquality = bool(value)

    def add_media_and_quality(self, rnumber):
        self.data += b'\x1B\x69\x7A' # ESC i z
        valid_flags = 0x80
        valid_flags |= (self._mtype is not None) << 1
        valid_flags |= (self._mwidth is not None) << 2
        valid_flags |= (self._mlength is not None) << 3
        valid_flags |= self._pquality << 6
        self.data += bytes([valid_flags])
        vals = [self._mtype, self._mwidth, self._mlength]
        self.data += b''.join(b'\x00' if val is None else val for val in vals)
        self.data += struct.pack('<L', rnumber)
        self.data += bytes([0 if self.page_number == 0 else 1])
        self.data += b'\x00'
        # INFO:  media/quality (1B 69 7A) --> found! (payload: 8E 0A 3E 00 D2 00 00 00 00 00)

    def add_autocut(self, autocut = False):
        if self.model not in cuttingsupport:
            self.unsupported("Trying to call add_autocut with a printer that doesn't support it")
            return
        self.data += b'\x1B\x69\x4D' # ESC i M
        self.data += bytes([autocut << 6])

    def add_cut_every(self, n=1):
        if self.model not in cuttingsupport:
            self.unsupported("Trying to call add_cut_every with a printer that doesn't support it")
            return
        self.data += b'\x1B\x69\x41' # ESC i A
        self.data += bytes([n & 0xFF])

    def add_expanded_mode(self):
        if self.model not in expandedmode:
            self.unsupported("Trying to set expanded mode (dpi/cutting at end) on a printer that doesn't support it")
            return
        if self.two_color_printing and not self.two_color_support:
            self.unsupported("Trying to set two_color_printing in expanded mode on a printer that doesn't support it.")
            return
        self.data += b'\x1B\x69\x4B' # ESC i K
        flags = 0x00
        flags |= self.cut_at_end << 3
        flags |= self.dpi_600 << 6
        flags |= self.two_color_printing << 0
        self.data += bytes([flags])

    def add_margins(self, dots=0x23):
        self.data += b'\x1B\x69\x64' # ESC i d
        self.data += struct.pack('<H', dots)

    def add_compression(self, compression=True):
        if self.model not in compressionsupport:
            self.unsupported("Trying to set compression on a printer that doesn't support it")
            return
        self._compression = compression
        self.data += b'\x4D' # M
        self.data += bytes([compression << 1])

    def get_pixel_width(self):
        try:
            nbpr = number_bytes_per_row[self.model]
        except:
            nbpr = number_bytes_per_row['default']
        return nbpr*8

    def add_raster_data(self, image, second_image=None):
        """ image: Pillow Image() """
        logger.info("raster_image_size: {0}x{1}".format(*image.size))
        if image.size[0] != self.get_pixel_width():
            fmt = 'Wrong pixel width: {}, expected {}'
            raise BrotherQLRasterError(fmt.format(image.size[0], self.get_pixel_width()))
        images = [image]
        if second_image:
            if image.size != second_image.size:
                fmt = "First and second image don't have the same dimesions: {} vs {}."
                raise BrotherQLRasterError(fmt.format(image.size, second_image.size))
            images.append(second_image)
        frames = []
        for image in images:
            image = image.transpose(Image.FLIP_LEFT_RIGHT)
            image = image.convert("1")
            frames.append(bytes(image.tobytes(encoder_name='raw')))
        frame_len = len(frames[0])
        row_len = images[0].size[0]//8
        start = 0
        file_str = BytesIO()
        while start + row_len <= frame_len:
            for i, frame in enumerate(frames):
                row = frame[start:start+row_len]
                if second_image:
                    file_str.write(b'\x77\x01' if i == 0 else b'\x77\x02')
                else:
                    file_str.write(b'\x67\x00')
                if self._compression:
                    row = packbits.encode(row)
                file_str.write(bytes([len(row)]))
                file_str.write(row)
            start += row_len
        self.data += file_str.getvalue()

    def add_print(self, last_page=True):
        if last_page:
            self.data += b'\x1A' # 0x1A = ^Z = SUB; here: EOF = End of File
        else:
            self.data += b'\x0C' # 0x0C = FF  = Form Feed




class BrotherQLError(Exception): pass
class BrotherQLUnsupportedCmd(BrotherQLError): pass
class BrotherQLUnknownModel(BrotherQLError): pass
class BrotherQLRasterError(BrotherQLError): pass

from .raster import BrotherQLRaster

from .brother_ql_create import create_label





from .generic import BrotherQLBackendGeneric


available_backends = [
  'pyusb',
  'network',
  'linux_kernel',
]

def guess_backend(descr):
    """ guess the backend from a given string descriptor for the device """
    if descr.startswith('usb://') or descr.startswith('0x'):
        return 'pyusb'
    elif descr.startswith('file://') or descr.startswith('/dev/usb/') or descr.startswith('lp'):
        return 'linux_kernel'
    elif descr.startswith('tcp://'):
        return 'network'
    else:
        raise ValueError('Cannot guess backend for given string descriptor: %s' % descr)
    

def backend_factory(backend_name):

    if backend_name == 'pyusb':
        from . import pyusb        as pyusb_backend
        list_available_devices = pyusb_backend.list_available_devices
        backend_class          = pyusb_backend.BrotherQLBackendPyUSB
    elif backend_name == 'linux_kernel':
        from . import linux_kernel as linux_kernel_backend
        list_available_devices = linux_kernel_backend.list_available_devices
        backend_class          = linux_kernel_backend.BrotherQLBackendLinuxKernel
    elif backend_name == 'network':
        from . import network      as network_backend
        list_available_devices = network_backend.list_available_devices
        backend_class          = network_backend.BrotherQLBackendNetwork
    else:
        raise NotImplementedError('Backend %s not implemented.' % backend_name)

    return {'list_available_devices': list_available_devices, 'backend_class': backend_class}



#!/usr/bin/env python

"""
Backend to support Brother QL-series printers via the linux kernel USB printer interface.
Works on Linux.
"""

import glob, os, time, select

from .generic import BrotherQLBackendGeneric

def list_available_devices():
    """
    List all available devices for the linux kernel backend

    returns: devices: a list of dictionaries with the keys 'string_descr' and 'instance': \
        [ {'string_descr': 'file:///dev/usb/lp0', 'instance': None}, ] \
        Instance is set to None because we don't want to open (and thus potentially block) the device here.
    """

    paths = glob.glob('/dev/usb/lp*')

    return [{'string_descr': 'file://' + path, 'instance': None} for path in paths]

class BrotherQLBackendLinuxKernel(BrotherQLBackendGeneric):
    """
    BrotherQL backend using the Linux Kernel USB Printer Device Handles
    """

    def __init__(self, device_specifier):
        """
        device_specifier: string or os.open(): string descriptor in the \
            format file:///dev/usb/lp0 or os.open() raw device handle.
        """

        self.read_timeout = 0.01
        # strategy : try_twice or select
        self.strategy = 'select'
        if isinstance(device_specifier, str):
            if device_specifier.startswith('file://'):
                device_specifier = device_specifier[7:]
            self.dev = os.open(device_specifier, os.O_RDWR)
        elif isinstance(device_specifier, int):
            self.dev = device_specifier
        else:
            raise NotImplementedError('Currently the printer can be specified either via an appropriate string or via an os.open() handle.')

        self.write_dev = self.dev
        self.read_dev  = self.dev

    def _write(self, data):
        os.write(self.write_dev, data)

    def _read(self, length=32):
        if self.strategy == 'try_twice':
            data = os.read(self.read_dev, length)
            if data:
                return data
            else:
                time.sleep(self.read_timeout)
                return os.read(self.read_dev, length)
        elif self.strategy == 'select':
            data = b''
            start = time.time()
            while (not data) and (time.time() - start < self.read_timeout):
                result, _, _ = select.select([self.read_dev], [], [], 0)
                if self.read_dev in result:
                    data += os.read(self.read_dev, length)
                if data: break
                time.sleep(0.001)
            if not data:
                # one last try if still no data:
                return os.read(self.read_dev, length)
            else:
                return data
        else:
            raise NotImplementedError('Unknown strategy')

    def _dispose(self):
        os.close(self.dev)



#!/usr/bin/env python

import sys, argparse, logging, struct, io, logging, sys, os, time
from pprint import pprint, pformat

from brother_ql.reader import OPCODES, chunker, merge_specific_instructions, interpret_response, match_opcode, hex_format
from brother_ql.backends import backend_factory, guess_backend

logger = logging.getLogger(__name__)

class BrotherQL_USBdebug(object):

    def __init__(self, dev, instructions_data, backend='linux_kernel'):

        be_cls = backend_factory(backend)['backend_class']
        self.be = be_cls(dev)

        self.sleep_time = 0.0
        self.sleep_before_read = 0.0
        self.continue_reading_for = 3.0
        self.start = time.time()
        self.interactive = False
        self.merge_specific_instructions = True
        if type(instructions_data) in (str,):
            with open(instructions_data, 'rb') as f:
                self.instructions_data = f.read()
        elif type(instructions_data) in (bytes,):
            self.instructions_data = instructions_data
        else:
            raise NotImplementedError('Only filename or bytes supported for instructions_data argument')
        response = self.be.read()
        if response:
            logger.warning('Received response before sending instructions: {}'.format(hex_format(response)))

    def continue_reading(self, seconds=3.0):
        start = time.time()
        while time.time() - start < seconds:
            data = self.be.read()
            if data != b'':
                global_time = time.time() - self.start
                print('TIME %.2f' % global_time)
                self.log_interp_response(data)
            time.sleep(0.001)

    def log_interp_response(self, data):
        try:
            interp_result = interpret_response(data)
            logger.info("Interpretation of the response: '{status_type}' (phase: {phase_type}), '{media_type}' {media_width}x{media_length} mm^2, errors: {errors}".format(**interp_result))
        except:
            logger.error("Couln't interpret response: %s", hex_format(data))

    def print_and_debug(self):

        self.continue_reading(0.2)

        instructions = chunker(self.instructions_data)
        instructions = merge_specific_instructions(instructions, join_preamble=True, join_raster=self.merge_specific_instructions)
        for instruction in instructions:
            opcode = match_opcode(instruction)
            opcode_def = OPCODES[opcode]
            cmd_name = opcode_def[0]
            hex_instruction = hex_format(instruction).split()
            if len(hex_instruction) > 100:
                hex_instruction = ' '.join(hex_instruction[0:70] + ['[...]'] + hex_instruction[-30:])
            else:
                hex_instruction = ' '.join(hex_instruction)
            logger.info("CMD {} FOUND. Instruction: {} ".format(cmd_name, hex_instruction))
            if self.interactive: input('Continue?')
            # WRITE
            self.be.write(instruction)
            # SLEEP BEFORE READ
            time.sleep(self.sleep_before_read)
            # READ
            response = self.be.read()
            #response += self.be.read()
            if response != b'':
                logger.info("Response from the device: {}".format(hex_format(response)))
                self.log_interp_response(response)
            # SLEEP BETWEEN INSTRUCTIONS
            time.sleep(self.sleep_time)

        self.continue_reading(self.continue_reading_for)

def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('file', help='The file to analyze')
    parser.add_argument('dev', help='The device to use. Can be usb://0x04f9:0x2015 or /dev/usb/lp0 for example')
    parser.add_argument('--sleep-time', type=float, help='time in seconds to sleep between instructions')
    parser.add_argument('--sleep-before-read', type=float, help='time in seconds to sleep before reading response')
    parser.add_argument('--continue-reading-for', type=float, help='continue reading after sending the last commands (time in seconds)')
    parser.add_argument('--interactive', action='store_true', help='interactive mode')
    parser.add_argument('--split-raster', action='store_true', help='even split preamble and raster instructions into single write operations')
    parser.add_argument('--debug', action='store_true', help='enable debug mode')
    args = parser.parse_args()

    # SETUP
    loglevel = logging.DEBUG if args.debug else logging.INFO
    logging.basicConfig(level=loglevel, format='%(levelname)s: %(message)s')

    try:
        backend = guess_backend(args.dev)
    except ValueError as e:
        parser.error(e.msg)

    br = BrotherQL_USBdebug(args.dev, args.file, backend=backend)
    if args.interactive: br.interactive = True
    if args.sleep_time: br.sleep_time = args.sleep_time
    if args.sleep_before_read: br.sleep_before_read = args.sleep_before_read
    if args.split_raster: br.merge_specific_instructions = False
    if args.continue_reading_for: br.continue_reading_for = args.continue_reading_for

    # GO
    br.print_and_debug()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""
Testing the packaged version of the Linux Kernel backend
"""

import argparse, logging, sys, time
from pprint import pprint

from brother_ql.backends import backend_factory, guess_backend, available_backends
from brother_ql.reader import interpret_response

logger = logging.getLogger(__name__)

def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('--backend', choices=available_backends, help='Forces the use of a specific backend')
    parser.add_argument('--list-printers', action='store_true', help='List the devices available with the selected --backend')
    parser.add_argument('--debug', action='store_true', help='Enable debugging output')
    parser.add_argument('instruction_file', nargs='?', help='file containing the instructions to be sent to the printer')
    parser.add_argument('device', metavar='DEVICE_STRING_DESCRIPTOR', nargs='?', help='String descriptor for specific device. If not specified, select first detected device')
    args = parser.parse_args()

    if args.list_printers and not args.backend:
        parser.error('Please specify the backend in order to list available devices.')

    if not args.list_printers and not args.instruction_file:
        parser.error("the following arguments are required: instruction_file")

    if args.instruction_file == '-':
        try:
            content = sys.stdin.buffer.read()
        except AttributeError:
            content = sys.stdin.read()
    else:
        with open(args.instruction_file, 'rb') as f:
            content = f.read()

    level = logging.DEBUG if args.debug else logging.INFO
    logging.basicConfig(level=level)
    if args.backend == 'network':
        logger.warning("The network backend doesn't supply any 'readback' functionality. No status reports will be received.")

    selected_backend = None
    if args.backend:
        selected_backend = args.backend
    else:
        try:
            selected_backend = guess_backend(args.device)
        except:
            logger.info("No backend stated. Selecting the default linux_kernel backend.")
            selected_backend = 'linux_kernel'

    be = backend_factory(selected_backend)
    list_available_devices = be['list_available_devices']
    BrotherQLBackend       = be['backend_class']

    if args.list_printers or not args.device:
        available_devices = list_available_devices()
        for ad in available_devices:
            result = {'model': 'unknown'}
            result.update(ad)
            logger.info("  Found a label printer: {string_descr}  (model: {model})".format(**result))

    if args.list_printers:
        for printer in available_devices:
            print(printer['string_descr'])
        sys.exit(0)

    string_descr = None
    if not args.device:
        "We need to search for available devices and select the first."
        if not available_devices:
            sys.exit("No printer found")
        string_descr = available_devices[0]['string_descr']
        print("Selecting first device %s" % string_descr)
    else:
        "A string descriptor for the device was given, let's use it."
        string_descr = args.device

    printer = BrotherQLBackend(string_descr)

    start = time.time()
    logger.info('Sending instructions to the printer. Total: %d bytes.', len(content))
    printer.write(content)
    if selected_backend == 'network':
        """ No need to wait for completion. The network backend doesn't support readback. """
        return
    printing_completed = False
    waiting_to_receive = False
    while time.time() - start < 10:
        data = printer.read()
        if not data:
            time.sleep(0.005)
            continue
        try:
            result = interpret_response(data)
        except ValueError:
            logger.error("TIME %.3f - Couln't understand response: %s", time.time()-start, data)
            continue
        logger.debug('TIME %.3f - result: %s', time.time()-start, result)
        if result['errors']:
            logger.error('Errors occured: %s', result['errors'])
        if result['status_type'] == 'Printing completed': printing_completed = True
        if result['status_type'] == 'Phase change' and result['phase_type'] == 'Waiting to receive': waiting_to_receive = True
        if printing_completed and waiting_to_receive:
            break
    if not (printing_completed and waiting_to_receive):
        logger.warning('Printing potentially not successful?')

if __name__ == "__main__": main()



#!/usr/bin/env python

from __future__ import division

import sys, argparse, logging

from PIL import Image
import PIL.ImageOps, PIL.ImageChops

from brother_ql.raster import BrotherQLRaster
from brother_ql.devicedependent import models, label_type_specs, ENDLESS_LABEL, DIE_CUT_LABEL, ROUND_DIE_CUT_LABEL
from brother_ql import BrotherQLError, BrotherQLUnsupportedCmd, BrotherQLUnknownModel
from brother_ql.image_trafos import filtered_hsv

try:
    stdout = sys.stdout.buffer
except:
    stdout = sys.stdout

try:
    unicode
except:
    unicode = str

logger = logging.getLogger(__name__)

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('image', help='The image file to create a label from.')
    parser.add_argument('outfile', nargs='?', type=argparse.FileType('wb'), default=stdout, help='The file to write the instructions to. Defaults to stdout.')
    parser.add_argument('--model', '-m', default='QL-500', help='The printer model to use. Check available ones with `brother_ql_info list-models`.')
    parser.add_argument('--label-size', '-s', default='62', help='The label size (and kind) to use. Check available ones with `brother_ql_info list-label-sizes`.')
    parser.add_argument('--rotate', '-r', choices=('0', '90', '180', '270'), default='auto', help='Rotate the image (counterclock-wise) by this amount of degrees.')
    parser.add_argument('--threshold', '-t', type=float, default=70.0, help='The threshold value (in percent) to discriminate between black and white pixels.')
    parser.add_argument('--dither', '-d', action='store_true', help='Enable dithering when converting the image to b/w. If set, --threshold is meaningless.')
    parser.add_argument('--compress', '-c', action='store_true', help='Enable compression (if available with the model). Takes more time but results in smaller file size.')
    parser.add_argument('--red', action='store_true', help='Create a label to be printed on black/red/white tape (only with QL-8xx series on DK-22251 labels). You must use this option when printing on black/red tape, even when not printing red.')
    parser.add_argument('--600dpi', action='store_true', dest='dpi_600', help='Print with 600x300 dpi available on some models. Provide your image as 600x600 dpi; perpendicular to the feeding the image will be resized to 300dpi.')
    parser.add_argument('--lq', action='store_false', dest='hq', help='Print with low quality (faster). Default is high quality.')
    parser.add_argument('--no-cut', dest='cut', action='store_false', help="Don't cut the tape after printing the label.")
    parser.add_argument('--loglevel', type=lambda x: getattr(logging, x), default=logging.WARNING, help='Set to DEBUG for verbose debugging output to stderr.')
    args = parser.parse_args()

    logging.basicConfig(level=args.loglevel)
    logging.getLogger("PIL.PngImagePlugin").setLevel(logging.WARNING)

    args.model = args.model.upper()

    try:
        qlr = BrotherQLRaster(args.model)
    except BrotherQLUnknownModel:
        sys.exit("Unknown model. Use the command   brother_ql_info list-models   to show available models.")

    try:
        label_type_specs[args.label_size]
    except ValueError:
        sys.exit("Unknown label_size. Check available sizes with the command   brother_ql_info list-label-sizes")

    qlr.exception_on_warning = True

    create_label(qlr, args.image, args.label_size, threshold=args.threshold, cut=args.cut, rotate=args.rotate, dither=args.dither, compress=args.compress, red=args.red, dpi_600=args.dpi_600, hq=args.hq)

    args.outfile.write(qlr.data)

def create_label(qlr, image, label_size, threshold=70, cut=True, dither=False, compress=False, red=False, **kwargs):

    label_specs = label_type_specs[label_size]
    dots_printable = label_specs['dots_printable']
    right_margin_dots = label_specs['right_margin_dots']
    device_pixel_width = qlr.get_pixel_width()
    rotate = kwargs.get('rotate', 'auto')
    if rotate != 'auto': rotate = int(rotate)
    dpi_600 = kwargs.get('dpi_600', False)
    hq = kwargs.get('hq', True)

    threshold = 100.0 - threshold
    threshold = min(255, max(0, int(threshold/100.0 * 255))) # from percent to pixel val

    if red and not qlr.two_color_support:
        raise BrotherQLUnsupportedCmd('Printing in red is not supported with the selected model.')

    if isinstance(image, Image.Image):
        im = image
    elif isinstance(image, (unicode, str)):
        im = Image.open(image)
    else:
        raise NotImplementedError("The image argument needs to be an Image() instance or the filename to an image.")

    if im.mode.endswith('A'):
        # place in front of white background and get red of transparency
        bg = Image.new("RGB", im.size, (255,255,255))
        bg.paste(im, im.split()[-1])
        im = bg
    elif im.mode == "P":
        # Convert GIF ("P") to RGB
        im = im.convert("RGB" if red else "L")

    if dpi_600:
        dots_expected = [el*2 for el in dots_printable]
    else:
        dots_expected = dots_printable

    if label_specs['kind'] == ENDLESS_LABEL:
        if rotate not in ('auto', 0):
            im = im.rotate(rotate, expand=True)
        if dpi_600:
            im = im.resize((im.size[0]//2, im.size[1]))
        if im.size[0] != dots_printable[0]:
            hsize = int((dots_printable[0] / im.size[0]) * im.size[1])
            im = im.resize((dots_printable[0], hsize), Image.ANTIALIAS)
            logger.warning('Need to resize the image...')
        if im.size[0] < device_pixel_width:
            new_im = Image.new(im.mode, (device_pixel_width, im.size[1]), (255,)*len(im.mode))
            new_im.paste(im, (device_pixel_width-im.size[0]-right_margin_dots, 0))
            im = new_im
    elif label_specs['kind'] in (DIE_CUT_LABEL, ROUND_DIE_CUT_LABEL):
        if rotate == 'auto':
            if im.size[0] == dots_expected[1] and im.size[1] == dots_expected[0]:
                im = im.rotate(90, expand=True)
        elif rotate != 0:
            im = im.rotate(rotate, expand=True)
        if im.size[0] != dots_expected[0] or im.size[1] != dots_expected[1]:
            raise ValueError("Bad image dimensions: %s. Expecting: %s." % (im.size, dots_expected))
        if dpi_600:
            im = im.resize((im.size[0]//2, im.size[1]))
        new_im = Image.new(im.mode, (device_pixel_width, dots_expected[1]), (255,)*len(im.mode))
        new_im.paste(im, (device_pixel_width-im.size[0]-right_margin_dots, 0))
        im = new_im

    if red:
        filter_h = lambda h: 255 if (h <  40 or h > 210) else 0
        filter_s = lambda s: 255 if s > 100 else 0
        filter_v = lambda v: 255 if v >  80 else 0
        red_im = filtered_hsv(im, filter_h, filter_s, filter_v)
        red_im = red_im.convert("L")
        red_im = PIL.ImageOps.invert(red_im)
        red_im = red_im.point(lambda x: 0 if x < threshold else 255, mode="1")

        filter_h = lambda h: 255
        filter_s = lambda s: 255
        filter_v = lambda v: 255 if v <  80 else 0
        black_im = filtered_hsv(im, filter_h, filter_s, filter_v)
        black_im = black_im.convert("L")
        black_im = PIL.ImageOps.invert(black_im)
        black_im = black_im.point(lambda x: 0 if x < threshold else 255, mode="1")
        black_im = PIL.ImageChops.subtract(black_im, red_im)
    else:
        im = im.convert("L")
        im = PIL.ImageOps.invert(im)

        if dither:
            im = im.convert("1", dither=Image.FLOYDSTEINBERG)
        else:
            im = im.point(lambda x: 0 if x < threshold else 255, mode="1")

    try:
        qlr.add_switch_mode()
    except BrotherQLUnsupportedCmd:
        pass
    qlr.add_invalidate()
    qlr.add_initialize()
    try:
        qlr.add_switch_mode()
    except BrotherQLUnsupportedCmd:
        pass
    qlr.add_status_information()
    tape_size = label_specs['tape_size']
    if label_specs['kind'] in (DIE_CUT_LABEL, ROUND_DIE_CUT_LABEL):
        qlr.mtype = 0x0B
        qlr.mwidth = tape_size[0]
        qlr.mlength = tape_size[1]
    else:
        qlr.mtype = 0x0A
        qlr.mwidth = tape_size[0]
        qlr.mlength = 0
    qlr.pquality = int(hq)
    qlr.add_media_and_quality(im.size[1])
    try:
        if cut:
            qlr.add_autocut(True)
            qlr.add_cut_every(1)
    except BrotherQLUnsupportedCmd:
        pass
    try:
        qlr.dpi_600 = dpi_600
        qlr.cut_at_end = cut
        qlr.two_color_printing = True if red else False
        qlr.add_expanded_mode()
    except BrotherQLUnsupportedCmd:
        pass
    qlr.add_margins(label_specs['feed_margin'])
    try:
        if compress: qlr.add_compression(True)
    except BrotherQLUnsupportedCmd:
        pass
    if red:
        qlr.add_raster_data(black_im, red_im)
    else:
        qlr.add_raster_data(im)
    qlr.add_print()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

import sys, argparse, logging

from brother_ql.reader import BrotherQLReader

def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('file', help='The file to analyze', type=argparse.FileType('rb'))
    parser.add_argument('--loglevel', type=lambda x: getattr(logging, x), default=logging.WARNING, help='The loglevel to apply')
    args = parser.parse_args()

    logging.basicConfig(stream=sys.stdout, format='%(levelname)s: %(message)s', level=args.loglevel)

    try:
        args.file = args.file.buffer
    except AttributeError:
        pass

    br = BrotherQLReader(args.file)
    br.analyse()

if __name__ == '__main__':
    main()
